package constructorsExercise3

import org.junit.Assert
import org.junit.FixMethodOrder
import org.junit.Test
import org.junit.runners.MethodSorters
import util.TIMEOUT

@FixMethodOrder(MethodSorters.NAME_ASCENDING)
class TestConstructorsExercise3 {
  class RobotHandler(fieldSize: Int, var x: Int, var y: Int) {
    val robot: Robot = Robot(fieldSize, x, y)
    val loggedMovements = StringBuilder()

    fun goRight(steps: Int) {
      robot.goRight(steps)
      loggedMovements.appendLine("right($steps)")
    }

    fun goLeft(steps: Int) {
      robot.goLeft(steps)
      loggedMovements.appendLine("left($steps)")

    }

    fun goDown(steps: Int) {
      robot.goDown(steps)
      loggedMovements.appendLine("down($steps)")

    }

    fun goUp(steps: Int) {
      robot.goUp(steps)
      loggedMovements.appendLine("up($steps)")
    }
  }

  private fun testMovements(string: String, fieldSize: Int, x: Int, y: Int,
                            movements: RobotHandler.() -> Unit) {
    val robotHandler = RobotHandler(fieldSize, x, y)
    robotHandler.movements()
    val actualString = robotHandler.robot.toString()
    val message = if (robotHandler.loggedMovements.isEmpty()) {
      "Initial location of the robot should be ($x, $y)"
    } else {
      "Starting location: ($x, $y), field size: $fieldSize.\n" +
        "Wrong location after:\n" + robotHandler.loggedMovements
    }
    Assert.assertEquals(message, string, actualString)
  }

  @Test(timeout = TIMEOUT)
  fun test1Initial() = testMovements("Robot(x=2, y=3)", 10, 2, 3) {}

  @Test(timeout = TIMEOUT)
  fun test2Right() = testMovements("Robot(x=3, y=3)", 10, 2, 3) {
    goRight(1)
  }

  @Test(timeout = TIMEOUT)
  fun test3Circle() = testMovements("Robot(x=2, y=3)", 10, 2, 3) {
    goRight(10)
    goLeft(10)
    goDown(10)
    goUp(10)
  }
}